package eu.DECIDEh2020.mcsla.editor.dual_listbox;
/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* <<licensing_schema_to_be_decided>> which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Majid Salehi Ghamsari Fraunhofer FOKUS
**Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*/
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import eu.DECIDEh2020.mcsla.editor.data.SlasData;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.select.SelectorComposer;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zk.ui.util.Clients;
import org.zkoss.zul.Groupbox;
import org.zkoss.zul.Label;
import org.zkoss.zul.ListModelList;
import org.zkoss.zul.Listbox;
import org.zkoss.zul.Radio;
import org.zkoss.zul.Radiogroup;
import org.zkoss.zul.Selectbox;
import org.zkoss.zul.Textbox;
import org.zkoss.zul.Window;

import eu.DECIDEh2020.mcsla.editor.JerseyClient;
import eu.DECIDEh2020.mcsla.beans.Mcsla;
import eu.DECIDEh2020.mcsla.beans.MicroserviceSLA;
import eu.DECIDEh2020.mcsla.beans.MicroserviceSLO;
import eu.DECIDEh2020.mcsla.beans.MicroserviceSQO;
import eu.DECIDEh2020.mcsla.beans.ProjectConfig;

public class DualListbox extends SelectorComposer<Component> {
	private static final long serialVersionUID = 8243942703081449079L;
	@Wire
	private Window win;
	@Wire
	Groupbox mcslaGroupbox;
	@Wire
	private Listbox candidateLbSQOs;
	@Wire
	private Listbox candidateLbSLOs;
	@Wire
	private Listbox candidateLbAggs;
	@Wire
	private Listbox chosenLbsqos;
	@Wire
	private Listbox chosenLbslos;
	@Wire
	private Listbox chosenLbaggs;
	@Wire
	private Label mcslaIDLabel;
	@Wire
	private Label gitRemoteLabel;
	@Wire
	private Radiogroup visibilityRadiogroup;
	@Wire
	private Selectbox visibilitySelectbox;
	@Wire
	private Textbox descriptionTextbox;

	private ListModelList<MicroserviceSQO> candidateSQOModel;
	private ListModelList<MicroserviceSQO> chosenSQODataModel;
	private ListModelList<MicroserviceSLO> candidateSLOModel;
	private ListModelList<MicroserviceSLO> chosenSLODataModel;
	private ListModelList<MicroserviceSLO> candidateAGGModel;
	private ListModelList<MicroserviceSLO> chosenAGGDataModel;
	private ListModelList<String> modelVisibilityList;
	private String[] visibilityList = { "public", "privat" };
	private ProjectConfig projectConfig;
	private Mcsla mcsla;

	public DualListbox() {
		// Executions.createComponents("/widgets/listbox/dual_listbox/v_dualListbox.zul",
		// this, null);
		// Selectors.wireComponents(this, this, false);
		// Selectors.wireEventListeners(this, this);
	}

	// This method will be called after all the child components under the component
	// which has the composer applied to it are created, so we can change components
	// attributes or even create other components in it.
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		reloadUI();
	}

	private void reloadUI() {
		chosenLbsqos.setModel(chosenSQODataModel = new ListModelList<MicroserviceSQO>());
		chosenLbslos.setModel(chosenSLODataModel = new ListModelList<MicroserviceSLO>());
		chosenLbaggs.setModel(chosenAGGDataModel = new ListModelList<MicroserviceSLO>());

		chosenSQODataModel.setMultiple(true);
		chosenSLODataModel.setMultiple(true);
		chosenAGGDataModel.setMultiple(true);

		setSQOModel(SlasData.getAllSQOs());
		setSLOModel(SlasData.getAllSLOs());
		setAGGModel(SlasData.getAllAGGs());

		JerseyClient jerseyClient = new JerseyClient();
		mcsla = jerseyClient.getMcsla();
		updateMcslaGUI();
		projectConfig = jerseyClient.getProjectConfig();
		gitRemoteLabel.setValue(projectConfig.getGitremotepath() + " (User:" + projectConfig.getGitusername() + ")");
	}

	public Mcsla getMcsla() {
		return mcsla;
	}

	/**
	 * @return the projectConfig
	 */
	public ProjectConfig getProjectConfig() {
		return projectConfig;
	}

	/**
	 * @param projectConfig
	 *            the projectConfig to set
	 */
	public void setProjectConfig(ProjectConfig projectConfig) {
		this.projectConfig = projectConfig;
	}

	public ListModelList<String> getVisibilityList() {
		modelVisibilityList = new ListModelList<String>(Arrays.asList(visibilityList));
		return modelVisibilityList;
	}

	@Listen("onSelect = #visibilitySelectbox")
	public void changeType() {
		Set<String> types = ((ListModelList<String>) modelVisibilityList).getSelection();
		String visibility = types.iterator().next();
		mcsla.setVisibility(visibility);
		showNotify("Visibility changed to: " + visibility, visibilitySelectbox);
	}

	@Listen("onClick = #saveButton")
	public void submit() {
		JerseyClient jerseyClient = new JerseyClient();
		List<MicroserviceSLO> slos = getChosenSLODataList();
		// System.out.println("DualListBox:submit:slos:" + slos.size());
		List<MicroserviceSLO> aggslos = getChosenAGGDataList();
		// System.out.println("DualListBox:submit:aggslos:" + aggslos.size());
		slos.addAll(aggslos);
		List<MicroserviceSQO> sqos = getChosenSQODataList();
		// System.out.println("DualListBox:submit:sqos:" + sqos.size());
		mcsla = jerseyClient.buildMCSLAwithslos(mcsla, slos, sqos);
		jerseyClient.updateMcsla(mcsla);
		showNotify("Your MCSLA is saved in Repository:" + projectConfig.getGitremotepath(), mcslaGroupbox);
	}

	@Listen("onClick = #aggDetailsButton")
	public void showAggDetails(Event event) {
		Set<MicroserviceSLO> set = candidateAGGModel.getSelection();
		showNotify("Your MCSLA aggDetailsButton." + set.size(), mcslaGroupbox);
	}

	@Listen("onClick = #loadButton")
	public void loadMcsla() {
		JerseyClient jerseyClient = new JerseyClient();
		mcsla = jerseyClient.getMcsla();
		updateMcslaGUI();
		showNotify("Get MCSLA from Repository:" + mcsla.getmCSLA_id(), mcslaIDLabel);
	}

	@Listen("onClick = #createNewButton")
	public void createNewMcsla() {
		setSQOModel(SlasData.getAllSQOs());
		setSLOModel(SlasData.getAllSLOs());
		setAGGModel(SlasData.getAllAGGs());

		JerseyClient jerseyClient = new JerseyClient();
		mcsla = jerseyClient.createMCSLAexample(null);
		updateMcslaGUI();
		showNotify("Created new MCSLA:" + mcsla.getmCSLA_id(), mcslaIDLabel);
	}

	private Set<MicroserviceSQO> buildSQOsSet(Mcsla mcsla) {
		List<MicroserviceSLA> slas = mcsla.getMicroservice_SLAs();
		List<MicroserviceSQO> sqoslist = new ArrayList<>();
		for (MicroserviceSLA sla : slas) {
			if (sla.getMicroservice_SQOs() != null)
				sqoslist.addAll(sla.getMicroservice_SQOs());
		}
		Set<MicroserviceSQO> sqosSet = new HashSet<MicroserviceSQO>(sqoslist);
		System.out.println("DualListBox:buildSQOsSet:slos:" + sqosSet.size());
		return sqosSet;
	}

	private Set<MicroserviceSLO> buildSLOsSet(Mcsla mcsla) {
		List<MicroserviceSLA> slas = mcsla.getMicroservice_SLAs();
		List<MicroserviceSLO> sloslist = new ArrayList<>();
		for (MicroserviceSLA sla : slas) {
			if (sla.getMicroservice_SLOs() != null)
				sloslist.addAll(sla.getMicroservice_SLOs());
		}
		Set<MicroserviceSLO> slosSet = new HashSet<MicroserviceSLO>(sloslist);
		System.out.println("DualListBox:buildSLOsSet:slos:" + slosSet.size());
		return slosSet;
	}

	private void updateMcslaGUI() {
		mcslaIDLabel.setValue(mcsla.getmCSLA_id().toString());
		descriptionTextbox.setValue(mcsla.getDescription());
		modelVisibilityList.addToSelection(mcsla.getVisibility());
		chooseSetSQO(buildSQOsSet(mcsla));
		chooseSetSLO(buildSLOsSet(mcsla));

	}

	@Listen("onChange = #descriptionTextbox")
	public void changeTitle() {
		String description = descriptionTextbox.getValue();
		mcsla.setDescription(description);
		showNotify("MCSLA Description changed to: " + description, descriptionTextbox);
	}

	@Listen("onCheck = #visibilityRadiogroup")
	public void changeVisibility() {
		Radio selectedItem = visibilityRadiogroup.getSelectedItem();
		// car.setAutoTransmission("automaticRadio".equals(selectedItem.getId()));
		showNotify("MCSLA Visibility changed to: " + selectedItem.getLabel(), visibilityRadiogroup.getParent());
	}

	private void showNotify(String msg, Component ref) {
		Clients.showNotification(msg, "info", ref, "end_center", 2000);
	}

	@Listen("onClick = #chooseAllBtnAgg")
	public void chooseAllItemAGG() {
		chosenAGGDataModel.addAll(candidateAGGModel);
		candidateAGGModel.clear();
	}

	@Listen("onClick = #chooseAllBtnSLO")
	public void chooseAllItemSLO() {
		chosenSLODataModel.addAll(candidateSLOModel);
		candidateSLOModel.clear();
	}

	@Listen("onClick = #chooseAllBtnSQO")
	public void chooseAllItemSQO() {
		chosenSQODataModel.addAll(candidateSQOModel);
		candidateSQOModel.clear();
	}

	// AGG
	@Listen("onClick = #chooseBtnAgg")
	public void chooseItemAGG() {
		// Events.postEvent(new ChooseEventAGG(this, chooseOneAGG()));
		chooseOneAGG();
	}

	// SLO
	@Listen("onClick = #chooseBtnSLO")
	public void chooseItemSLO() {
		// Events.postEvent(new ChooseEventSLO(this, chooseOneSLO()));
		chooseOneSLO();
	}

	// SQO
	@Listen("onClick = #chooseBtnSQO")
	public void chooseItemSQO() {
		// Events.postEvent(new ChooseEventSQO(this, chooseOneSQO()));
		chooseOneSQO();
	}

	// AGG
	private Set<MicroserviceSLO> chooseOneAGG() {
		Set<MicroserviceSLO> set = candidateAGGModel.getSelection();
		chosenAGGDataModel.addAll(set);
		candidateAGGModel.removeAll(set);
		return set;
	}

	// SLO
	private Set<MicroserviceSLO> chooseOneSLO() {
		Set<MicroserviceSLO> set = candidateSLOModel.getSelection();
		chosenSLODataModel.addAll(set);
		candidateSLOModel.removeAll(set);
		return set;
	}

	private Set<MicroserviceSQO> chooseOneSQO() {
		Set<MicroserviceSQO> set = candidateSQOModel.getSelection();
		chosenSQODataModel.addAll(set);
		candidateSQOModel.removeAll(set);
		return set;
	}

	// FIX this
	private Set<MicroserviceSQO> chooseSetSQO(Set<MicroserviceSQO> set) {
		System.out.println("DualListbox:chooseSetSQO:" + set.size());
		if (set.size() == 0)
			return set;

		setSQOModel(SlasData.getAllSQOs());
		// chosenSQODataModel.clear();
		chosenSQODataModel.addAll(set);

		List<MicroserviceSQO> list2removeList = new ArrayList<MicroserviceSQO>();
		List<MicroserviceSQO> candidateSQOList = candidateSQOModel.getInnerList();
		for (Iterator<MicroserviceSQO> iterator = set.iterator(); iterator.hasNext();) {
			MicroserviceSQO microserviceSQO1 = (MicroserviceSQO) iterator.next();
			for (Iterator<MicroserviceSQO> iterator2 = candidateSQOList.iterator(); iterator2.hasNext();) {
				MicroserviceSQO microserviceSQO2 = (MicroserviceSQO) iterator2.next();
				if (microserviceSQO1.getMicroservice_SQO_id().equals(microserviceSQO2.getMicroservice_SQO_id())) {
					list2removeList.add(microserviceSQO2);
				}
			}
		}
		boolean b = candidateSQOModel.removeAll(list2removeList);
		System.out.println("DualListbox:chooseSetSQO:" + b);

		return set;
	}

	// FIX this
	private Set<MicroserviceSLO> chooseSetSLO(Set<MicroserviceSLO> set) {
		System.out.println("DualListbox:chooseSetSLO:" + set.size());
		if (set.size() == 0)
			return set;
		setSLOModel(SlasData.getAllSLOs());
		// chosenSLODataModel.clear();
		chosenSLODataModel.addAll(set);

		List<MicroserviceSLO> list2removeList = new ArrayList<MicroserviceSLO>();
		List<MicroserviceSLO> candidateSLOList = candidateSLOModel.getInnerList();
		for (Iterator<MicroserviceSLO> iterator = set.iterator(); iterator.hasNext();) {
			MicroserviceSLO microserviceSLO1 = (MicroserviceSLO) iterator.next();
			for (Iterator<MicroserviceSLO> iterator2 = candidateSLOList.iterator(); iterator2.hasNext();) {
				MicroserviceSLO microserviceSLO2 = (MicroserviceSLO) iterator2.next();
				if (microserviceSLO1.getMicroservice_SLO_id().equals(microserviceSLO2.getMicroservice_SLO_id())) {
					list2removeList.add(microserviceSLO2);
				}
			}
		}
		boolean b = candidateSLOModel.removeAll(list2removeList);
		System.out.println("DualListbox:chooseSetSLO:" + b);
		b = candidateAGGModel.removeAll(list2removeList);
		System.out.println("DualListbox:chooseSetSLO:" + b);
		return set;
	}

	/**
	 * @return current chosen SLO data list
	 */
	public List<MicroserviceSLO> getChosenSLODataList() {
		return new ArrayList<MicroserviceSLO>(chosenSLODataModel);
	}

	/**
	 * @return current chosen SQO data list
	 */
	public List<MicroserviceSQO> getChosenSQODataList() {
		return new ArrayList<MicroserviceSQO>(chosenSQODataModel);
	}

	/**
	 * @return current chosen AGG data list
	 */
	public List<MicroserviceSLO> getChosenAGGDataList() {
		return new ArrayList<MicroserviceSLO>(chosenAGGDataModel);
	}

	/**
	 * Set new candidate AGG ListModelList.
	 * 
	 * @param candidate
	 *            is the data of candidate list model
	 */
	public void setAGGModel(List<MicroserviceSLO> candidate) {
		candidateLbAggs.setModel(this.candidateAGGModel = new ListModelList<MicroserviceSLO>(candidate));
		this.candidateAGGModel.setMultiple(true);
		chosenAGGDataModel.clear();
	}

	/**
	 * Set new candidate SLO ListModelList.
	 * 
	 * @param candidate
	 *            is the data of candidate list model
	 */
	public void setSLOModel(List<MicroserviceSLO> candidate) {
		candidateLbSLOs.setModel(this.candidateSLOModel = new ListModelList<MicroserviceSLO>(candidate));
		this.candidateSLOModel.setMultiple(true);
		chosenSLODataModel.clear();
	}

	/**
	 * Set new candidate SQO ListModelList.
	 * 
	 * @param candidate
	 *            is the data of candidate list model
	 */
	public void setSQOModel(List<MicroserviceSQO> candidate) {
		candidateLbSQOs.setModel(this.candidateSQOModel = new ListModelList<MicroserviceSQO>(candidate));
		this.candidateSQOModel.setMultiple(true);
		chosenSQODataModel.clear();
	}

	@Listen("onClick = #removeAllBtnAgg")
	public void unchooseAllAgg() {
		candidateAGGModel.addAll(chosenAGGDataModel);
		chosenAGGDataModel.clear();
	}

	@Listen("onClick = #removeAllBtnSLO")
	public void unchooseAllSLO() {
		candidateSLOModel.addAll(chosenSLODataModel);
		chosenSLODataModel.clear();
	}

	@Listen("onClick = #removeAllBtnSQO")
	public void unchooseAllSQO() {
		candidateSQOModel.addAll(chosenSQODataModel);
		chosenSQODataModel.clear();
	}

	@Listen("onClick = #removeBtnAgg")
	public void unchooseItemAGG() {
		// Events.postEvent(new ChooseEventAGG(this, unchooseOneAGG()));
		unchooseOneAGG();
	}

	@Listen("onClick = #removeBtnSLO")
	public void unchooseItemSLO() {
		// Events.postEvent(new ChooseEventSLO(this, unchooseOneSLO()));
		unchooseOneSLO();
	}

	@Listen("onClick = #removeBtnSQO")
	public void unchooseItemSQO() {
		// Events.postEvent(new ChooseEventSQO(this, unchooseOneSQO()));
		unchooseOneSQO();
	}

	private Set<MicroserviceSLO> unchooseOneAGG() {
		Set<MicroserviceSLO> set = chosenAGGDataModel.getSelection();
		candidateAGGModel.addAll(set);
		chosenAGGDataModel.removeAll(set);
		return set;
	}

	private Set<MicroserviceSLO> unchooseOneSLO() {
		Set<MicroserviceSLO> set = chosenSLODataModel.getSelection();
		candidateSLOModel.addAll(set);
		chosenSLODataModel.removeAll(set);
		return set;
	}

	private Set<MicroserviceSQO> unchooseOneSQO() {
		Set<MicroserviceSQO> set = chosenSQODataModel.getSelection();
		candidateSQOModel.addAll(set);
		chosenSQODataModel.removeAll(set);
		return set;
	}

	// Customized Event AGG
	public class ChooseEventAGG extends Event {
		private static final long serialVersionUID = -7554906383953342976L;

		public ChooseEventAGG(Component target, Set<MicroserviceSLO> data) {
			super("onChoose", target, data);
		}
	}

	// Customized Event SLO
	public class ChooseEventSLO extends Event {
		private static final long serialVersionUID = -7444906383953342976L;

		public ChooseEventSLO(Component target, Set<MicroserviceSLO> data) {
			super("onChoose", target, data);
		}
	}

	// Customized Event SQO
	public class ChooseEventSQO extends Event {
		private static final long serialVersionUID = -7334906383953342976L;

		public ChooseEventSQO(Component target, Set<MicroserviceSQO> data) {
			super("onChoose", target, data);
		}
	}
}