package eu.DECIDEh2020.mcsla.service.git;

/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* <<licensing_schema_to_be_decided>> which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Majid Salehi Ghamsari Fraunhofer FOKUS
**Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*/
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.UUID;

import org.eclipse.jgit.api.AddCommand;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.PushCommand;
import org.eclipse.jgit.api.errors.CanceledException;
import org.eclipse.jgit.api.errors.ConcurrentRefUpdateException;
import org.eclipse.jgit.api.errors.DetachedHeadException;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.InvalidConfigurationException;
import org.eclipse.jgit.api.errors.InvalidRemoteException;
import org.eclipse.jgit.api.errors.JGitInternalException;
import org.eclipse.jgit.api.errors.NoFilepatternException;
import org.eclipse.jgit.api.errors.NoHeadException;
import org.eclipse.jgit.api.errors.NoMessageException;
import org.eclipse.jgit.api.errors.RefNotFoundException;
import org.eclipse.jgit.api.errors.WrongRepositoryStateException;
import org.eclipse.jgit.internal.storage.file.FileRepository;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.PushResult;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import eu.DECIDEh2020.architect.plugin.models.AppDescriptor;
import eu.DECIDEh2020.mcsla.beans.Mcsla;
import eu.DECIDEh2020.mcsla.service.configuration.AppConfig;
import eu.DECIDEh2020.mcsla.service.service.ConfigFileService;

public class GitControl {

	private String localPath, remotePath;
	private Repository localRepo;
	private Git git;
	private CredentialsProvider cp;
	private String username;
	private String userpassword;

	public GitControl() throws IOException {
		System.out.println("GitControl:.....");
		this.localPath = System.getProperty("java.io.tmpdir") + File.separator + UUID.randomUUID();
		ConfigFileService configService = getConfigService();
		this.remotePath = configService.getGitremotepath();
		this.username = configService.getGitusername();
		this.userpassword = configService.getGituserpassword();

		this.localRepo = new FileRepository(localPath + "/.git");
		cp = new UsernamePasswordCredentialsProvider(this.username, this.userpassword);
		git = new Git(localRepo);
	}

	public GitControl(String localPath, String remotePath, String username, String password) throws IOException {
		System.out.println("GitControl:.....");
		this.localPath = localPath;
		this.remotePath = remotePath;
		this.username = username;
		this.userpassword = password;

		this.localRepo = new FileRepository(localPath + "/.git");
		cp = new UsernamePasswordCredentialsProvider(this.username, this.userpassword);
		git = new Git(localRepo);
	}

	public void pushMcslaToRepo(Mcsla mcsla)
			throws IOException, NoFilepatternException, GitAPIException, FileNotFoundException, NoHeadException,
			NoMessageException, ConcurrentRefUpdateException, WrongRepositoryStateException, InvalidRemoteException,
			InvalidConfigurationException, DetachedHeadException, CanceledException, RefNotFoundException {
		System.out.println("GitControl:pushMcslaToRepo:.....");
		// Clone repository
		try {
			cloneRepo();

		} catch (Exception e) {
			e.printStackTrace();
		}

		/*
		 * try (FileWriter writer = new FileWriter(localPath + "/MCSLA" + ".json")) {
		 * Gson gsonb = new GsonBuilder().setPrettyPrinting().create();
		 * gsonb.toJson(mcsla, writer);
		 * 
		 * } catch (IOException e) { e.printStackTrace(); }
		 */

		Gson gson = new Gson();
		String APPLICATIONDES = localPath + "/DECIDE" + ".json";
		AppDescriptor appDescriptor = gson.fromJson(new FileReader(APPLICATIONDES), AppDescriptor.class);
		System.out.println("GitControl:pushToRepo:appDescriptor:" + appDescriptor.getIDString());
		// appDescriptor.setApp_mcsla(mcsla.mCSLA_id); //old
		appDescriptor.setApp_mcsla(mcsla);
		// System.out.println("GitControl:pushToRepo:mCSLA_id:" +
		// appDescriptor.getApp_mcsla());

		try (FileWriter writer = new FileWriter(APPLICATIONDES)) {
			Gson gsonb = new GsonBuilder().setPrettyPrinting().create();
			gsonb.toJson(appDescriptor, writer);

		} catch (IOException e) {
			e.printStackTrace();
		}

		// Add files to repository
		addToRepo();
		// Commit with a custom message
		commitToRepo("Modified from MCSLA editor, updated the MCSLA:" + mcsla.mCSLA_id);
		// Push commits
		pushToRepo();
		// Pull
		pullFromRepo();
	}

	public Mcsla getMcslaFromRepo()
			throws IOException, NoFilepatternException, GitAPIException, FileNotFoundException, NoHeadException,
			NoMessageException, ConcurrentRefUpdateException, WrongRepositoryStateException, InvalidRemoteException,
			InvalidConfigurationException, DetachedHeadException, CanceledException, RefNotFoundException {
		System.out.println("GitControl:getMcslaFromRepo:.....");
		try {
			cloneRepo();

		} catch (Exception e) {
			e.printStackTrace();
		}

		Gson gson = new Gson();
		String APPLICATIONDES = localPath + "/DECIDE" + ".json";
		AppDescriptor appDescriptor = gson.fromJson(new FileReader(APPLICATIONDES), AppDescriptor.class);
		System.out.println("GitControl:getMcslaFromRepo:appDescriptor:" + appDescriptor.getIDString());
		// appDescriptor.setApp_mcsla(mcsla.mCSLA_id); //old
		String jsonMcslaInString = gson.toJson(appDescriptor.getApp_mcsla());
		Mcsla mcsla = gson.fromJson(jsonMcslaInString, Mcsla.class);
		System.out.println("GitControl:getMcslaFromRepo:mCSLA_id:" + mcsla.getmCSLA_id() + " Description:"
				+ mcsla.getDescription());
		return mcsla;
	}

	public void cloneRepo() throws IOException, NoFilepatternException, GitAPIException {
		Git.cloneRepository().setURI(remotePath)
				.setCredentialsProvider(new UsernamePasswordCredentialsProvider(this.username, this.userpassword))
				.setDirectory(new File(localPath)).call();

	}

	public void addToRepo() throws IOException, NoFilepatternException, GitAPIException {
		AddCommand add = git.add();
		add.addFilepattern(".").call();
	}

	public void commitToRepo(String message) throws IOException, NoHeadException, NoMessageException,
			ConcurrentRefUpdateException, JGitInternalException, WrongRepositoryStateException, GitAPIException {
		git.commit().setMessage(message).call();
	}

	public void pushToRepo() throws IOException, JGitInternalException, InvalidRemoteException, GitAPIException {
		PushCommand pc = git.push();
		pc.setCredentialsProvider(cp).setForce(true).setPushAll();
		try {
			Iterator<PushResult> it = pc.call().iterator();
			if (it.hasNext()) {
				System.out.println(it.next().toString());
			}
		} catch (InvalidRemoteException e) {
			e.printStackTrace();
		}
	}

	public void pullFromRepo()
			throws IOException, WrongRepositoryStateException, InvalidConfigurationException, DetachedHeadException,
			InvalidRemoteException, CanceledException, RefNotFoundException, NoHeadException, GitAPIException {
		git.pull().setCredentialsProvider(new UsernamePasswordCredentialsProvider(this.username, this.userpassword))
				.call();
	}

	private ConfigFileService getConfigService() {
		AbstractApplicationContext context = new AnnotationConfigApplicationContext(AppConfig.class);
		ConfigFileService configService = (ConfigFileService) context.getBean("configFileService");
		context.close();
		return configService;
	}

}
